package ru.dbotthepony.mc.otm;

import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.InterModComms;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.event.lifecycle.InterModEnqueueEvent;
import net.minecraftforge.fml.event.lifecycle.InterModProcessEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import ru.dbotthepony.mc.otm.capability.AndroidCapability;
import ru.dbotthepony.mc.otm.capability.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.client.AndroidGui;
import ru.dbotthepony.mc.otm.client.EventHandler;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.matter.MatterRegistry;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;

import java.util.stream.Collectors;

// The value here should match an entry in the META-INF/mods.toml file
@Mod(OverdriveThatMatters.MOD_ID)
public class OverdriveThatMatters {
	// Directly reference a log4j logger.
	public static final String MOD_ID = "overdrive_that_matters";
	public static final Logger LOGGER = LogManager.getLogger();

	public static AndroidGui ANDROID_GUI;

	public static CreativeModeTab CREATIVE_TAB;

	public OverdriveThatMatters() {
		// Register the setup method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setup);
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setupClient);
		// Register the enqueueIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::enqueueIMC);
		// Register the processIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::processIMC);

		// Register ourselves for server and other game events we are interested in
		MinecraftForge.EVENT_BUS.register(this);
		MinecraftForge.EVENT_BUS.register(AndroidCapabilityPlayer.class);
		MinecraftForge.EVENT_BUS.register(AndroidCapability.class);
		MinecraftForge.EVENT_BUS.register(MatterGrid.class);
		MinecraftForge.EVENT_BUS.register(MatterRegistry.class);

		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Items.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Blocks.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.BlockEntities.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Menus.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.AndroidFeatures.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.AndroidResearch.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Stats.class);

		FMLJavaModLoadingContext.get().getModEventBus().addListener(AndroidCapability::registerEffects);

		// LOGGER.info("Registered event handlers");

		Registry.createRegistry();

		CREATIVE_TAB = new CreativeModeTab("otm") {
			@Override
			public ItemStack makeIcon() {
				return new ItemStack(Registry.Items.BATTERY_CREATIVE, 1);
			}
		};
	}

	private void setup(final FMLCommonSetupEvent event) {
		// some preinit code
		MatteryCapability.register();
		// LOGGER.info("Registered capabilities");
		MatteryNetworking.register();
		// LOGGER.info("Registered network");

		MatterRegistry.registerInitialItems();
	}

	private void setupClient(final FMLClientSetupEvent event) {
		ANDROID_GUI = new AndroidGui();
		MinecraftForge.EVENT_BUS.register(ANDROID_GUI);
		MinecraftForge.EVENT_BUS.register(AndroidGui.class);
		MinecraftForge.EVENT_BUS.register(EventHandler.class);
	}

	private void enqueueIMC(final InterModEnqueueEvent event) {
		// some example code to dispatch IMC to another mod
		InterModComms.sendTo("examplemod", "helloworld", () -> { LOGGER.info("Hello world from the MDK"); return "Hello world";});
	}

	private void processIMC(final InterModProcessEvent event) {
		// some example code to receive and process InterModComms from other mods
		LOGGER.info("Got IMC {}", event.getIMCStream().
				map(m->m.messageSupplier().get()).
				collect(Collectors.toList()));
	}
}
