package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import ru.dbotthepony.mc.otm.android.AndroidFeature;
import ru.dbotthepony.mc.otm.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.util.Optional;

public interface IAndroidCapability extends IMatteryEnergyStorage, INBTSerializable<CompoundTag> {
	void tick();
	void tickClient();
	LivingEntity getEntity();

	<T extends AndroidFeature> T addFeature(@Nonnull AndroidFeatureType<T> feature);
	boolean removeFeature(@Nonnull AndroidFeatureType<?> feature);
	boolean hasFeature(@Nullable AndroidFeatureType<?> feature);
	boolean hasFeatureLevel(@Nullable AndroidFeatureType<?> feature, int level);
	<T extends AndroidFeature> T getFeature(AndroidFeatureType<T> feature);

	@Nonnull
	default <T extends AndroidFeature> Optional<T> getFeatureO(AndroidFeatureType<T> feature) {
		var get = getFeature(feature);
		return get != null ? Optional.of(get) : Optional.empty();
	}

	default boolean isAndroid() {
		return true;
	}
	default void onHurt(LivingHurtEvent event) {}

	@Nonnull
	ItemStack getBatteryItemStack();
	void setBatteryItemStack(@Nonnull ItemStack stack);

	void invalidateNetworkState(); // tell capability that player forgot everything, and everything needs to be re-networked

	void setEnergy(BigDecimal value);
	void setMaxEnergy(BigDecimal value);
}
