package ru.dbotthepony.mc.otm.client;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.screens.DeathScreen;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.effect.MobEffects;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraftforge.client.event.GuiOpenEvent;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.client.gui.ForgeIngameGui;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;

import java.math.BigDecimal;
import java.util.Optional;

public class AndroidGui {
	private static final Minecraft mc = Minecraft.getInstance();
	private final ForgeIngameGui gui = mc.gui instanceof ForgeIngameGui ? (ForgeIngameGui) mc.gui : null;

	public static final ResourceLocation PLAYER_GUI_LOCATION = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/player_gui.png");

	private IAndroidCapability last_state;

	@SubscribeEvent
	public static void onOpenGUI(GuiOpenEvent event) {
		if (!(event.getGui() instanceof DeathScreen screen))
			return;

		if (mc.player == null)
			return;

		mc.player.getCapability(MatteryCapability.ANDROID).ifPresent(cap -> {
			if (cap.isAndroid())
				screen.title = new TranslatableComponent("otm.death_reason");
		});
	}

	@SubscribeEvent
	public void onLayerRenderEvent(RenderGameOverlayEvent.PreLayer event) {
		if (event.getOverlay() != ForgeIngameGui.FOOD_LEVEL_ELEMENT && event.getOverlay() != ForgeIngameGui.AIR_LEVEL_ELEMENT)
			return;

		LocalPlayer ply = mc.player;

		if (ply == null)
			return;

		if (
				ply.getVehicle() instanceof LivingEntity ||
				mc.options.hideGui ||
				!mc.gameMode.canHurtPlayer() ||
				!(mc.getCameraEntity() instanceof Player)
		)
			return;

		Optional<IAndroidCapability> lazy = ply.getCapability(MatteryCapability.ANDROID).resolve();
		IAndroidCapability android = null;

		if (lazy.isPresent()) {
			android = lazy.get();
		} else if (!ply.isAlive()) {
			android = last_state;
		}

		if (android != null && android.isAndroid() && mc.gui instanceof ForgeIngameGui gui) {
			event.setCanceled(true);
			last_state = android;

			if (event.getOverlay() == ForgeIngameGui.AIR_LEVEL_ELEMENT) {
				return;
			}

			int y_offset = ply.hasEffect(MobEffects.HUNGER) ? 18 : 0;

			float level;

			if (android.getMaxBatteryLevel().compareTo(BigDecimal.ZERO) == 0) {
				level = 0f;
			} else {
				level = android.getBatteryLevel().divide(android.getMaxBatteryLevel(), MatteryCapability.ROUND_RULES).floatValue();

				if (level >= 0.98f)
					level = 1f;
			}

			gui.setupOverlayRenderState(true, false);

			RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
			RenderSystem.setShaderTexture(0, PLAYER_GUI_LOCATION);

			int width = event.getWindow().getGuiScaledWidth();
			int height = event.getWindow().getGuiScaledHeight();

			int left = width / 2 + 10;
			int top = height - gui.right_height;

			gui.right_height += 10;

			// Stack, x, y, blitOffset?, (float) image_x, (float) image_y, rect_x, rect_y, total_image_width, total_image_height
			// Stack, x, y, image_x, image_y, rect_x, rect_y
			gui.blit(event.getMatrixStack(), left, top, 0, y_offset, 80, 9);

			int shift_left = (int) Math.ceil(level * 79f - 0.5f);
			gui.blit(event.getMatrixStack(), left + 79 - shift_left, top, 79 - shift_left, y_offset + 9, shift_left, 9);
		}
	}
}
